=== modified file 'NEWS'
--- NEWS	2006-08-01 07:39:26 +0000
+++ NEWS	2006-08-14 04:11:10 +0000
@@ -1,11 +1,14 @@
 IN DEVELOPMENT
-
+  
   IMPROVEMENTS:
 
-   * The revision specifier "revno:" is extended to accept the syntax
-     revno:N:branch. For example,
-     revno:42:http://bazaar-vcs.org/bzr/bzr.dev/ means revision 42 in
-     bzr.dev.  (Matthieu Moy)
+  BUG FIXES:
+
+  INTERNALS:
+
+bzr 0.9.0  2006-08-11
+
+  SURPRISES:
 
    * The hard-coded built-in ignore rules have been removed. There are
      now two rulesets which are enforced. A user global one in 
@@ -18,6 +21,17 @@
      running 'bzr ignore --old-default-rules'.
      (Robert Collins, Martin Pool, John Arbash Meinel)
 
+   * 'branches.conf' has been changed to 'locations.conf', since it can apply
+     to more locations than just branch locations.
+     (Aaron Bentley)
+   
+  IMPROVEMENTS:
+
+   * The revision specifier "revno:" is extended to accept the syntax
+     revno:N:branch. For example,
+     revno:42:http://bazaar-vcs.org/bzr/bzr.dev/ means revision 42 in
+     bzr.dev.  (Matthieu Moy)
+
    * Tests updates to ensure proper URL handling, UNICODE support, and
      proper printing when the user's terminal encoding cannot display 
      the path of a file that has been versioned.
@@ -80,10 +94,6 @@
      the same way you would apply another branch.
      (John Arbash Meinel, Aaron Bentley)
   
-   * 'branches.conf' has been changed to 'locations.conf', since it can apply
-     to more locations than just branch locations.
-     (Aaron Bentley)
-   
    * 'bzr whoami' can now be used to set your identity from the command line,
      for a branch or globally.  (Robey Pointer)
 
@@ -211,6 +221,9 @@
 
     * Use the right suffixes for loading plugins (John Arbash Meinel, #51810)
 
+    * Fix Branch.get_parent() to handle the case when the parent is not 
+      accessible (John Arbash Meinel, #52976)
+
   INTERNALS:
 
     * Combine the ignore rules into a single regex rather than looping over

=== modified file 'bzr'
--- bzr	2006-07-30 07:02:22 +0000
+++ bzr	2006-08-14 04:11:10 +0000
@@ -70,7 +70,7 @@
                      "\n")
     raise
 
-if bzrlib.version_info[:3] != (0, 9, 0):
+if bzrlib.version_info[:3] != (0, 10, 0):
     sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
             "This may indicate an installation problem.\n"
             "bzrlib from %s is version %r\n"

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2006-07-20 21:51:21 +0000
+++ bzrlib/__init__.py	2006-08-14 04:11:10 +0000
@@ -33,7 +33,6 @@
 del locale
 
 __copyright__ = "Copyright 2005, 2006 Canonical Development Ltd."
-__version__ = version_string = '0.9'
 
 # same format as sys.version_info: "A tuple containing the five components of
 # the version number: major, minor, micro, releaselevel, and serial. All
@@ -42,7 +41,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (0, 9, 0, 'dev', 0)
+version_info = (0, 10, 0, 'dev', 0)
 
 if version_info[3] == 'final':
     version_string = '%d.%d.%d' % version_info[:3]

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-07-20 14:01:13 +0000
+++ bzrlib/branch.py	2006-08-10 02:23:35 +0000
@@ -534,9 +534,13 @@
                 rev = self.repository.get_revision(revision_id)
                 new_history = rev.get_history(self.repository)[1:]
         destination.set_revision_history(new_history)
-        parent = self.get_parent()
-        if parent:
-            destination.set_parent(parent)
+        try:
+            parent = self.get_parent()
+        except errors.InaccessibleParent, e:
+            mutter('parent was not accessible to copy: %s', e)
+        else:
+            if parent:
+                destination.set_parent(parent)
 
     @needs_read_lock
     def check(self):
@@ -1170,7 +1174,10 @@
             # turn it into a url
             if parent.startswith('/'):
                 parent = urlutils.local_path_to_url(parent.decode('utf8'))
-            return urlutils.join(self.base[:-1], parent)
+            try:
+                return urlutils.join(self.base[:-1], parent)
+            except errors.InvalidURLJoin, e:
+                raise errors.InaccessibleParent(parent, self.base)
         return None
 
     def get_push_location(self):

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2006-08-05 22:33:36 +0000
+++ bzrlib/delta.py	2006-08-14 04:09:45 +0000
@@ -16,7 +16,7 @@
 
 from bzrlib.inventory import InventoryEntry
 from bzrlib.trace import mutter
-from bzrlib.symbol_versioning import deprecated_function, zero_ten
+from bzrlib.symbol_versioning import deprecated_function, zero_nine
 
 
 class TreeDelta(object):
@@ -143,7 +143,7 @@
             show_list(self.unchanged)
 
 
-@deprecated_function(zero_ten)
+@deprecated_function(zero_nine)
 def compare_trees(old_tree, new_tree, want_unchanged=False,
                   specific_files=None, extra_trees=None,
                   require_versioned=False):

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-08-09 00:50:42 +0000
+++ bzrlib/errors.py	2006-08-14 04:09:45 +0000
@@ -312,6 +312,14 @@
         self.function = function
 
 
+class InaccessibleParent(PathError):
+    """Parent not accessible given base %(base)s and relative path %(path)s"""
+
+    def __init__(self, path, base):
+        PathError.__init__(self, path)
+        self.base = base
+
+
 class NoRepositoryPresent(BzrNewError):
     """No repository present: %(path)r"""
     def __init__(self, bzrdir):

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-08-05 22:33:36 +0000
+++ bzrlib/repository.py	2006-08-14 04:09:45 +0000
@@ -40,11 +40,15 @@
         zero_nine, 
         )
 from bzrlib.testament import Testament
-from bzrlib.trace import mutter, note
+from bzrlib.trace import mutter, note, warning
 from bzrlib.tsort import topo_sort
 from bzrlib.weave import WeaveFile
 
 
+# Old formats display a warning, but only once
+_deprecation_warning_done = False
+
+
 class Repository(object):
     """Repository holding history for one or more branches.
 
@@ -188,6 +192,7 @@
         self.control_weaves = control_store
         # TODO: make sure to construct the right store classes, etc, depending
         # on whether escaping is required.
+        self._warn_if_deprecated()
 
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, 
@@ -680,6 +685,14 @@
         result.check()
         return result
 
+    def _warn_if_deprecated(self):
+        global _deprecation_warning_done
+        if _deprecation_warning_done:
+            return
+        _deprecation_warning_done = True
+        warning("Format %s for %s is deprecated - please use 'bzr upgrade' to get better performance"
+                % (self._format, self.bzrdir.transport.base))
+
 
 class AllInOneRepository(Repository):
     """Legacy support - the repository behaviour for all-in-one branches."""
@@ -792,7 +805,6 @@
                                                 _revision_store,
                                                 control_store,
                                                 text_store)
-
         dir_mode = self.control_files._dir_mode
         file_mode = self.control_files._file_mode
 
@@ -827,6 +839,10 @@
 class KnitRepository(MetaDirRepository):
     """Knit format repository."""
 
+    def _warn_if_deprecated(self):
+        # This class isn't deprecated
+        pass
+
     def _inventory_add_lines(self, inv_vf, revid, parents, lines):
         inv_vf.add_lines_with_ghosts(revid, parents, lines)
 
@@ -999,6 +1015,9 @@
     _formats = {}
     """The known formats."""
 
+    def __str__(self):
+        return "<%s>" % self.__class__.__name__
+
     @classmethod
     def find_format(klass, a_bzrdir):
         """Return the format for the repository object in a_bzrdir."""

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-08-09 15:19:54 +0000
+++ bzrlib/tests/__init__.py	2006-08-14 04:09:45 +0000
@@ -1235,6 +1235,12 @@
              test_suite_factory=None,
              lsprof_timed=None):
     """Run the whole test suite under the enhanced runner"""
+    # XXX: Very ugly way to do this...
+    # Disable warning about old formats because we don't want it to disturb
+    # any blackbox tests.
+    from bzrlib import repository
+    repository._deprecation_warning_done = True
+
     global default_transport
     if transport is None:
         transport = default_transport

=== modified file 'bzrlib/tests/blackbox/test_exceptions.py'
--- bzrlib/tests/blackbox/test_exceptions.py	2006-06-15 05:49:38 +0000
+++ bzrlib/tests/blackbox/test_exceptions.py	2006-08-11 10:03:44 +0000
@@ -19,6 +19,8 @@
 import os
 import sys
 
+from bzrlib import bzrdir, repository
+
 from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.errors import NotBranchError
 
@@ -35,3 +37,19 @@
     # register (and unregister) it from tests that want to touch it.
     #
     # TODO: Some kind of test for the feature of invoking pdb
+    
+
+class TestDeprecationWarning(TestCaseInTempDir):
+
+    def test_repository_deprecation_warning(self):
+        """Old formats give a warning"""
+        # the warning's normally off for testing but we reenable it
+        repository._deprecation_warning_done = False
+        try:
+            os.mkdir('foo')
+            bzrdir.BzrDirFormat5().initialize('foo')
+            out, err = self.run_bzr("status", "foo")
+            self.assertContainsRe(self._get_log(), "bzr upgrade")
+        finally:
+            repository._deprecation_warning_done = True
+

=== modified file 'bzrlib/tests/branch_implementations/__init__.py'
--- bzrlib/tests/branch_implementations/__init__.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/branch_implementations/__init__.py	2006-08-10 02:23:35 +0000
@@ -42,6 +42,7 @@
         'bzrlib.tests.branch_implementations.test_bound_sftp',
         'bzrlib.tests.branch_implementations.test_branch',
         'bzrlib.tests.branch_implementations.test_break_lock',
+        'bzrlib.tests.branch_implementations.test_http',
         'bzrlib.tests.branch_implementations.test_locking',
         'bzrlib.tests.branch_implementations.test_parent',
         'bzrlib.tests.branch_implementations.test_permissions',

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2006-08-10 02:23:35 +0000
@@ -181,14 +181,37 @@
         br_b = wt_a.bzrdir.open_branch().sprout(repo_b.bzrdir, revision_id='1')
         self.assertEqual(br_b.last_revision(), '1')
 
+    def get_parented_branch(self):
+        wt_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/one'])
+        wt_a.add(['one'])
+        wt_a.commit('commit one', rev_id='1')
+
+        branch_b = wt_a.bzrdir.sprout('b', revision_id='1').open_branch()
+        self.assertEqual(wt_a.branch.base, branch_b.get_parent())
+        return branch_b
+
     def test_clone_branch_nickname(self):
         # test the nick name is preserved always
         raise TestSkipped('XXX branch cloning is not yet tested..')
 
     def test_clone_branch_parent(self):
         # test the parent is preserved always
-        raise TestSkipped('XXX branch cloning is not yet tested..')
-        
+        branch_b = self.get_parented_branch()
+        repo_c = self.make_repository('c')
+        branch_b.repository.copy_content_into(repo_c)
+        branch_c = branch_b.clone(repo_c.bzrdir)
+        self.assertNotEqual(None, branch_c.get_parent())
+        self.assertEqual(branch_b.get_parent(), branch_c.get_parent())
+
+        # We can also set a specific parent, and it should be honored
+        random_parent = 'http://bazaar-vcs.org/path/to/branch'
+        branch_b.set_parent(random_parent)
+        repo_d = self.make_repository('d')
+        branch_b.repository.copy_content_into(repo_d)
+        branch_d = branch_b.clone(repo_d.bzrdir)
+        self.assertEqual(random_parent, branch_d.get_parent())
+
     def test_sprout_branch_nickname(self):
         # test the nick name is reset always
         raise TestSkipped('XXX branch sprouting is not yet tested..')

=== added file 'bzrlib/tests/branch_implementations/test_http.py'
--- bzrlib/tests/branch_implementations/test_http.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_http.py	2006-07-31 22:53:44 +0000
@@ -0,0 +1,72 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test branch responses when accessed over http"""
+
+import os
+
+from bzrlib import branch, errors
+from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
+from bzrlib.transport.http import HttpServer
+
+
+class HTTPBranchTests(TestCaseWithBranch):
+    """Tests that use an HTTP server against each branch implementation"""
+
+    def setUp(self):
+        super(HTTPBranchTests, self).setUp()
+        self.transport_readonly_server = HttpServer
+
+    def get_parent_and_child(self):
+        os.makedirs('parent/path/to')
+        wt_a = self.make_branch_and_tree('parent/path/to/a')
+        self.build_tree(['parent/path/to/a/one'])
+        wt_a.add(['one'])
+        wt_a.commit('commit one', rev_id='1')
+
+        os.makedirs('child/path/to')
+        branch_b = wt_a.bzrdir.sprout('child/path/to/b', revision_id='1').open_branch()
+        self.assertEqual(wt_a.branch.base, branch_b.get_parent())
+
+        return wt_a.branch, branch_b
+
+    def test_get_parent_invalid(self):
+        self.get_parent_and_child()
+
+        # Now change directory, and access the child through http
+        os.chdir('child/path/to')
+        branch_b = branch.Branch.open(self.get_readonly_url('b'))
+        self.assertRaises(errors.InaccessibleParent, branch_b.get_parent)
+
+    def test_clone_invalid_parent(self):
+        self.get_parent_and_child()
+
+        # Now change directory, and access the child through http
+        os.chdir('child/path/to')
+        branch_b = branch.Branch.open(self.get_readonly_url('b'))
+
+        branch_c = branch_b.bzrdir.clone('c').open_branch()
+        self.assertEqual(None, branch_c.get_parent())
+
+    def test_sprout_invalid_parent(self):
+        self.get_parent_and_child()
+
+        # Now change directory, and access the child through http
+        os.chdir('child/path/to')
+        branch_b = branch.Branch.open(self.get_readonly_url('b'))
+
+        branch_c = branch_b.bzrdir.sprout('c').open_branch()
+        self.assertEqual(branch_b.base, branch_c.get_parent())

=== modified file 'bzrlib/tests/branch_implementations/test_parent.py'
--- bzrlib/tests/branch_implementations/test_parent.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/branch_implementations/test_parent.py	2006-08-10 02:23:35 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005 by Canonical Ltd
+# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -48,7 +48,6 @@
         self.assertEqual(None, b.get_parent())
 
         b.set_parent('../other_branch')
-        cwd = getcwd()
 
         self.assertEqual(local_path_to_url('../other_branch'), b.get_parent())
         path = local_path_to_url('../yanb')
@@ -60,7 +59,7 @@
 
         self.assertRaises(bzrlib.errors.InvalidURL, b.set_parent, u'\xb5')
         b.set_parent(escape(u'\xb5'))
-        self.assertEqual('%C2%B5', 
+        self.assertEqual('%C2%B5',
             b.control_files.get('parent').read().strip('\n'))
 
         self.assertEqual(b.base + '%C2%B5', b.get_parent())
@@ -74,3 +73,17 @@
             b.control_files.put('parent', cStringIO.StringIO('/local/abs/path'))
             self.assertEqual('file:///local/abs/path', b.get_parent())
 
+    def test_get_invalid_parent(self):
+        b = self.make_branch('.')
+
+        cwd = getcwd()
+        n_dirs = len(cwd.split('/'))
+
+        # Force the relative path to be something invalid
+        # This should attempt to go outside the filesystem
+        path = ('../'*(n_dirs+5)) + 'foo'
+        b.control_files.put('parent', cStringIO.StringIO(path))
+
+        # With an invalid branch parent, just return None
+        self.assertRaises(bzrlib.errors.InaccessibleParent, b.get_parent)
+

=== modified file 'bzrlib/tests/repository_implementations/__init__.py'
--- bzrlib/tests/repository_implementations/__init__.py	2006-07-31 16:29:15 +0000
+++ bzrlib/tests/repository_implementations/__init__.py	2006-08-11 10:03:44 +0000
@@ -17,7 +17,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-"""BzrDir implementation tests for bzr.
+"""Repository implementation tests for bzr.
 
 These test the conformance of all the repository variations to the expected API.
 Specific tests for individual formats are in the tests/test_repository.py file 

